import { PluginCreator } from 'postcss';

/**
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

declare const plugin: PluginCreator<Options>;
type Options = {
    /**
     * A mapping from `font-family` to its font type (`sans-serif`, `serif`, or
     * `mono`).
     *
     * A `@font-face` rule is only processed if its `font-family` is in this
     * mapping.
     */
    readonly fontTypes: Readonly<Record<string, FontType>>;
    /**
     * An optional function that transforms a `@font-face` rule's `src` `url`
     * value to a file system path to the font file.
     *
     * The path is resolved relative to `process.cwd()`.
     */
    readonly srcUrlToFilename?: (url: string) => string;
};
type FontType = `sans-serif` | `serif` | `mono`;

export { FontType, Options, plugin as default };
